#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"

#include "spec_ktsal_en.h"

// masks for status register
#define M_HS_GOOD              (0x0100)
#define M_VALID_2              (0x0080)
#define M_VALID_1              (0x0040)
#define M_AC_2                 (0x0020)
#define M_DC12_2               (0x0010)
#define M_DC24_2               (0x0008)
#define M_AC_1                 (0x0004)
#define M_DC12_1               (0x0002)
#define M_DC24_1               (0x0001)

int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    TSYNC_HsStatusObj  status;


    /* If invalid number of arguments... */
    if ((argc != 2) && (argc != 3))
    {
        printf(" Usage: HS_GetStatus <device index>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    // Send Get Instances transaction
    err = TSYNC_HS_getStatus(hnd, &status);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /*
      * On definition of the environment variable SPEC_KTSAL_EN
      * to true, print out the information in the format used
      * for the equivalent set's command line.
      */
    if ((getenv(SPEC_KTSAL_EN) != NULL) &&
            *getenv(SPEC_KTSAL_EN) == *ENV_TRUE)
    {
        printf("%x  %d %d %d %3.2f %2.3f %2.3f  %d %d %d %3.2f %2.3f %2.3f\n",
            status.statusReg,
            status.sled1.fan_en,
            status.sled1.fan_pwm,
            status.sled1.fan_speed,
            status.sled1.temp,
            status.sled1.voltage,
            status.sled1.current,
            status.sled2.fan_en,
            status.sled2.fan_pwm,
            status.sled2.fan_speed,
            status.sled2.temp,
            status.sled2.voltage,
            status.sled2.current);
    }

    printf("\n HS Status: \n");
    printf("  Status Reg: %x\n", status.statusReg);
    printf("    Hot Swap Good: %s\n", (status.statusReg & M_HS_GOOD) ? "True" : "False" );
    printf("  Sled 1:\n");
    printf("    Valid:      %s\n", (status.statusReg & M_VALID_1) ? "True" : "False" );
    printf("    AC:         %s\n", (status.statusReg & M_AC_1)    ? "True" : "False" );
    printf("    12V DC:     %s\n", (status.statusReg & M_DC12_1)  ? "True" : "False" );
    printf("    24V DC:     %s\n", (status.statusReg & M_DC24_1)  ? "True" : "False" );
    printf("    Fan Enable: %d\n", status.sled1.fan_en);
    printf("    Fam PWM:    %d\n", status.sled1.fan_pwm);
    printf("    Fan Speed:  %d\n", status.sled1.fan_speed);
    printf("    Temp:       %3.2f\n", status.sled1.temp);
    printf("    Voltage:    %2.3f\n", status.sled1.voltage);
    printf("    Current:    %2.3f\n", status.sled1.current);
    printf("  Sled 2:\n");
    printf("    Valid:      %s\n", (status.statusReg & M_VALID_2) ? "True" : "False" );
    printf("    AC:         %s\n", (status.statusReg & M_AC_2)    ? "True" : "False" );
    printf("    12V DC:     %s\n", (status.statusReg & M_DC12_2)  ? "True" : "False" );
    printf("    24V DC:     %s\n", (status.statusReg & M_DC24_2)  ? "True" : "False" );
    printf("    Fan Enable: %d\n", status.sled2.fan_en);
    printf("    Fam PWM:    %d\n", status.sled2.fan_pwm);
    printf("    Fan Speed:  %d\n", status.sled2.fan_speed);
    printf("    Temp:       %3.2f\n", status.sled2.temp);
    printf("    Voltage:    %2.3f\n", status.sled2.voltage);
    printf("    Current:    %2.3f\n", status.sled2.current);

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
