#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "tsync.h"
#include "tsync_example.h"


int main (int argc, char *argv[])
{
    TSYNC_BoardHandle  hnd;
    int                rc;
    int                ret     = 0;
    int                devIdx;
    char              *devName = DEFAULT_DEV;
    char               fullDevName[32];
    char              *pgmname = argv[0];
    TSYNC_ERROR        err     = TSYNC_SUCCESS;
    unsigned int       index;
    TSYNC_StrConfObj   conf;


    /* If invalid number of arguments... */
    if (argc != 12)
    {
        printf(" Usage: STR_SetConf <device index> <index> <serial number> <latitude> <longitude> <altitude> <EE axis> <NN axis> <UU axis> <Geolocation mode> <sensitivity level>\n");
        return (1);
    }

    /* Construct the TSYNC/TSAT device name */
    devIdx = atoi(argv[1]);
    (void)sprintf( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TSYNC_open(&hnd, fullDevName)) != TSYNC_SUCCESS )
    {
        /*
         * if errors, close it
         */
        printf("%s: !Could not open <%s> : rc <%d>\n",
                        pgmname, fullDevName, rc);
        return (1);
    }

    /* Build Set Offset message */
    index = atoi(argv[2]);

    memset (conf.serialnb, 0, sizeof(conf.serialnb));
    strncpy (conf.serialnb, argv[3], strlen (argv[3]));
    conf.pos.lat   = atof(argv[4]) * DEG_TO_RAD;
    conf.pos.lon   = atof(argv[5]) * DEG_TO_RAD;
    conf.pos.alt   = atof(argv[6]);
    conf.ee_axis   = atof(argv[7]);
    conf.nn_axis   = atof(argv[8]);
    conf.uu_axis   = atof(argv[9]);
    conf.geoMode   = atoi(argv[10]);
    conf.sensLevel = atoi(argv[11]);

    printf("\n  STR (%d) Conf: %s %lf %lf %lf %lf %lf %lf %d %d\n",
        (index), (conf.serialnb), (conf.pos.lat), (conf.pos.lon), (conf.pos.alt),
        (conf.ee_axis),(conf.nn_axis),(conf.uu_axis),
        (conf.geoMode), (conf.sensLevel));

    // Send Set Offset transaction
    err = TSYNC_STR_setConf(hnd, index, conf);

    if (err != TSYNC_SUCCESS)
    {
        printf("  Error: %s.\n", tsync_strerror(err));
        return (1);
    }

    /* Close the TSYNC/TSAT device */
    if ( (rc = TSYNC_close(hnd)) != TSYNC_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n",
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);

} /* End - main() */
